mod power;
mod lib8;
mod render;
mod task;
mod time;
mod geometry;
mod platform;
mod animations;
mod mappings;
mod buffers;

use crate::platform::{DefaultBoard, Board};
use crate::task::{Task, Scheduler};
use crate::render::{Surfaces, Renderer};
use crate::geometry::Rectangle;

fn main() {
    let mut board: DefaultBoard = Board::take();

    log::info!("Board: {}", core::any::type_name_of_val(&board));

    let output = board.output();
    log::info!("Output: {}", core::any::type_name_of_val(&output));

    let mut surfaces = board.surfaces();
    log::info!("Surface implementation: {}", core::any::type_name_of_val(&output));

    log::info!("Creating animations");
    let mut tasks: Vec<Box<dyn Task + 'static>> = vec![
        Box::new(animations::IdleTask::new(&mut surfaces)),
        Box::new(animations::TestPattern::new(surfaces.new_surface(&Rectangle::everything()).unwrap())),
    ];

    tasks.append(&mut board.system_tasks());
    tasks.push(Box::new(Renderer::new(output, surfaces)));

    let mut runner = Scheduler::new(tasks);

    log::info!("Runner ready: {:?}", runner);

    log::info!("Ready to rock and roll");
    loop {
        runner.tick();
    }
}
